/*global define,window,navigation*/

define(['testsuite/qunit', 'jquery', 'de_epages/mediagallery/filemanager/navigation-view', 'de_epages/mediagallery/filemanager/folder-collection', '$ready!'], function (QUnit, $, NavigationView, FolderCollection) {
    'use strict';

    QUnit.module('de_epages/mediagallery/filemanager/navigation-view', {
        setup: function () {
            var filemanager = $('<div class="filemanager" />').appendTo('#qunit-fixture'),
                data = [{
                    "currentGallerySize": "1344",
                    "children": [{
                        "files": [{
                            "width": null,
                            "extension": ".swf",
                            "name": "promo.swf",
                            "path": "/promo.swf",
                            "height": null,
                            "directory": "/MediaGallery",
                            "modified": 1200994146,
                            "size": 67089,
                            "fullpath": "/WebRoot/Store/Shops/DemoShop/MediaGallery/promo.swf",
                            "selectable": 0,
                            "mimetype": "application/x-shockwave-flash"
                        }, {
                            "width": "240",
                            "extension": ".gif",
                            "name": "StartPagePromotion.gif",
                            "path": "/StartPagePromotion.gif",
                            "height": "113",
                            "directory": "/MediaGallery",
                            "modified": 1200994146,
                            "size": 14453,
                            "preview": "/WebRoot/Store/Shops/DemoShop/MediaGallery/StartPagePromotion.gif",
                            "fullpath": "/WebRoot/Store/Shops/DemoShop/MediaGallery/StartPagePromotion.gif",
                            "selectable": 1,
                            "mimetype": "image/gif"
                        }],
                        "children": [{
                            "files": [],
                            "children": [],
                            "metadata": {
                                "displayname": "neu",
                                "type": "MediaGallery",
                                "path": "/neu"
                            },
                            "data": "neu",
                            "state": "open"
                        }],
                        "metadata": {
                            "displayname": "MediaGallery",
                            "type": "MediaGallery",
                            "path": ""
                        },
                        "data": "MediaGallery",
                        "state": "open"
                    }, {
                        "files": [],
                        "children": [{
                            "files": [],
                            "children": [],
                            "metadata": {
                                "displayname": "Kunst, Kultur und Bildung",
                                "type": "Header",
                                "path": "/StyleGroups/Branches/StyleGroups/ArtAndEducation"
                            },
                            "data": "ArtAndEducation",
                            "state": null
                        }, {
                            "files": [],
                            "children": [],
                            "metadata": {
                                "displayname": "Auto und Transport",
                                "type": "Header",
                                "path": "/StyleGroups/Branches/StyleGroups/Automotive"
                            },
                            "data": "Automotive",
                            "state": null
                        }],
                        "metadata": {
                            "displayname": "Kopfzeilenbilder",
                            "type": "Header",
                            "path": "/StyleGroups/Branches/StyleGroups"
                        },
                        "data": "HeaderImages",
                        "state": "closed"
                    }, {
                        "files": [],
                        "children": [{
                            "files": [],
                            "children": [],
                            "metadata": {
                                "displayname": "Kunst, Kultur und Bildung",
                                "type": "Background",
                                "path": "/StyleGroups/Branches/StyleGroups/ArtAndEducation"
                            },
                            "data": "ArtAndEducation",
                            "state": null
                        }],
                        "metadata": {
                            "displayname": "Hintergrundbilder",
                            "type": "Background",
                            "path": "/StyleGroups/Branches/StyleGroups"
                        },
                        "data": "BackgroundImages",
                        "state": "closed"
                    }, {
                        "files": [],
                        "children": [{
                            "files": [],
                            "children": [],
                            "metadata": {
                                "displayname": "Gebäude",
                                "type": "ImageDB",
                                "path": "/ImageDBGroups/Buildings"
                            },
                            "data": "Buildings",
                            "state": null
                        }],
                        "metadata": {
                            "displayname": "{ImageDB}",
                            "type": "ImageDB",
                            "path": "ImageDBGroups"
                        },
                        "data": "ImageDB",
                        "state": "closed"
                    }],
                    "maxGallerySize": "102400"
                }],
                folders = new FolderCollection(data[0].children),
                model;

            window.navigation = new NavigationView({collection: folders, rootFolderData: data[0].children, currentFolder: 'MediaGallery'});

            filemanager.append(navigation.render().el);

            model = navigation.getOpenFolder();

            QUnit.strictEqual(
                $('.ep-mediagallery-navigation').length,
                1,
                'Create navigation view and append it to filemanager'
            );

            model
                .on('addTreeNode', function (event) {
                    QUnit.strictEqual(
                        event.id,
                        'MediaGallery',
                        'Trigger event addTreeNode on tree model'
                    );
                })
                .on('renameTreeNode', function (event) {
                    QUnit.strictEqual(
                        event.prevID,
                        'MediaGallery',
                        'Trigger event renameTreeNode on tree model'
                    );
                })
                .on('removeTreeNode', function (event) {
                    QUnit.strictEqual(
                        event.id,
                        'HeaderImages',
                        'Trigger event removeTreeNode on tree model'
                    );
                })
                .on('setActiveNode', function (event) {
                    QUnit.strictEqual(
                        event.id,
                        'HeaderImages',
                        'Trigger event setActiveNode on tree model'
                    );
                });

            navigation
                .on('openFolder', function (event) {
                    QUnit.strictEqual(
                        event.type,
                        'openFolder',
                        'Trigger event openFolder on navigation view'
                    );
                })
                .on('folderEditable', function (event) {
                    QUnit.strictEqual(
                        event.editable,
                        true,
                        'Trigger event folderEditable on navigation view'
                    );
                });
        },
        teardown: function () {
            $('.ep-mediagallery-navigation').remove();

            QUnit.strictEqual(
                $('.ep-mediagallery-navigation').length,
                0,
                'Remove navigation view from filemanager container'
            );
        }
    });

    QUnit.test('view methods', 9 + 2, function () {
        $.each(['initialize', 'render', 'addTreeNode', 'renameTreeNode', 'removeTreeNode', 'setActiveNode', 'triggerOpenFolder', 'triggerFolderEditable', 'changeState'], function (i, method) {
            QUnit.strictEqual(
                $.isFunction(navigation[method]),
                true,
                method + ' is a method of navigation view'
            );
        });
    });

    QUnit.test('view functionality', 2 + 8, function () {
        var model = navigation.getOpenFolder();

        QUnit.strictEqual(
            model.get('data'),
            'MediaGallery',
            'MediaGallery is the currently opened folder'
        );

        // START: tests for the following methods can be found in the setup
        navigation.addTreeNode({
            data: {},
            id: 'MediaGallery',
            path: ''
        });

        navigation.renameTreeNode({
            model: model,
            prevID: 'MediaGallery'
        });


        navigation.removeTreeNode({
            id: 'HeaderImages'
        });

        navigation.setActiveNode({
            id: 'HeaderImages'
        });

        navigation.triggerOpenFolder({
            type: 'openFolder'
        });

        navigation.triggerFolderEditable({
            editable: true
        });
        // END

        navigation.changeState();

        QUnit.strictEqual(
            navigation.getOpenFolder(),
            undefined,
            'MediaGallery folders state was changed to closed'
        );
    });

});