define("de_epages/molliepayments/ui/mbosettings", [
    "jquery",
    "ep/ajax",

    'jquery/tmpl',
    "ep/ui/validate",
    "ep/fn/busy",
    'jquery/metaparse',
    "$ready!"
], function ($, ep) {

        var mboSettingsObj = {},

            /*
             * event handler for profile select field
             */
            onChangeProfileId = function(element) {
                if (element.value !== "-1") {
                    getEnabledPaymentMethods();
                } else {
                    // reset payment methods list
                    clearPaymentMethods();
                }
            },

            /*
             * event handler for test mode select field
             */
            onChangeTestMode = function(element) {
                if (mboSettingsObj.profileSelect.value === "-1") {
                    return;
                }

                //clearPaymentMethods();
                getEnabledPaymentMethods();
            },

            /*
             * event handler for buttons to traverse profiles
             */
            onClickGetProfileList = function(nextOrPrevious) {
                var fromProfileId;

                // reset fields
                clearPaymentMethods();
                removeSelectOptions(mboSettingsObj.profileSelect);

                // hide buttons
                mboSettingsObj.$previousMollieProfiles && mboSettingsObj.$previousMollieProfiles.hide();
                mboSettingsObj.$nextMollieProfiles && mboSettingsObj.$nextMollieProfiles.hide();

                // get payment profiles
                fromProfileId = (nextOrPrevious === "next") ? mboSettingsObj.nextProfileId : mboSettingsObj.previousProfileId;
                getProfilesList(fromProfileId);
            },

            /*
             * error handler
             */
            onError = function(caller) {
                return function(errorMessage, response) {
                    alert(errorMessage);
                    console.log("Error by caller: " + caller);
                };
            },

            /*
             * initiates an AJAX request and executes a callback
             * function after success to update the UI with the response
             * or an error callback if request fails
             */
            doRequest = function(requestData, callbackSuccess, callbackError) {
                ep("body").busy("show");

                ep.ajax({
                    "data": requestData,
                    "type": "GET",
                    "dataType": "json"
                }).done(function (response) {
                    // execute callback function to update the UI with the response
                    callbackSuccess(response);
                }).fail(function( response ) {
                    callbackError(mboSettingsObj.errorMessages.connectionError , response);
                }).always(function() {
                    ep("body").busy("hide");
                });
            },

            /*
             * empties, resets and disables a select element
             */
            removeSelectOptions = function(selectElement, options) {
                var disableElement = options && options.disableElement !== undefined ? options.disableElement : true;
                selectElement.value = "-1";
                selectElement.disabled = disableElement;

                for (var i = selectElement.options.length - 1; i >= 0; i--) {
                    if (selectElement.options[i].value !== "-1") {
                        selectElement.remove(i);
                    }
                }
            },

            /*
             * prepares and initiates a request to get all enabled payment methods for a profile ID
             */
            getEnabledPaymentMethods = function() {
                // prepare request data
                var requestData = {
                    "ChangeAction": "GetMolliePaymentsMethods",
                    "ObjectID": mboSettingsObj.objectId,
                    "ProfileId": mboSettingsObj.profileSelect.value,
                    "IsInTestMode": mboSettingsObj.testModeSelect.value
                };

                // initiate request
                doRequest(requestData, onPaymentMethodsResponse, onError("getEnabledPaymentMethods"));
            },

            /*
             * prepares and initiates a request to get profiles
             */
            getProfilesList = function(profileIdFrom) {
                // prepare data
                var requestData = {
                    "ChangeAction": "GetMolliePaymentsProfiles",
                    "ObjectID": mboSettingsObj.objectId,
                    "profileIdFrom": profileIdFrom ? profileIdFrom : ""
                };

                // initiate request
                doRequest(requestData, onProfilesListResponse, onError("getProfilesList"));
            },

            /*
             * callback function => executed after successful request to get payment methods
             */
            onPaymentMethodsResponse = function(response) {
                // uncomment next line to mock empty response
                // response.methods = [];

                if (!response || typeof response.methods !== "object") {
                    var errorThrower = onError("onPaymentMethodsResponse");
                    errorThrower(mboSettingsObj.errorMessages.unknownError, response);
                    return;
                }

                // populate payment methods to page
                addPaymentMethodsToPage(response.methods);
            },

            /*
             * callback function => executed after successful request to get profiles
             */
            onProfilesListResponse = function(response) {
                if (!response || typeof response.profiles !== "object") {
                    var errorThrower = onError("onProfilesListResponse");
                    errorThrower(mboSettingsObj.errorMessages.unknownError, response);
                    return;
                }

                // populates profile select field
                addProfilesToSelect(response.profiles);

                // handle previous/next buttons if necessary
                if (response.previous) {
                    mboSettingsObj.previousProfileId = response.previous;
                    mboSettingsObj.$previousMollieProfiles && mboSettingsObj.$previousMollieProfiles.show();
                }
                if (response.next) {
                    mboSettingsObj.nextProfileId = response.next;
                    mboSettingsObj.$nextMollieProfiles && mboSettingsObj.$nextMollieProfiles.show();
                }
            },

            clearPaymentMethods = function() {
                var methodsListElem = null;
                if ( mboSettingsObj.methodsList ) {
                    methodsListElem = $(mboSettingsObj.methodsList);
                }

                if ( methodsListElem ) {
                    methodsListElem.find('.NoMethodsFound').hide().closest('.DetailsRow').hide();
                    methodsListElem.find('.SepaWarning').show().closest('.DetailsRow').show();
                    methodsListElem.find('.MethodsEntryCheckBox, .MethodsEntryDesc').remove();
                }
            },

            /*
             * method to populate payment methods to the page with handed over methods
             */
            addPaymentMethodsToPage = function(methods) {
                methods = methods ? methods : [];

                clearPaymentMethods();

                var methodsListElem = null;
                if ( mboSettingsObj.methodsList ) {
                    methodsListElem = $(mboSettingsObj.methodsList);
                }

                for (var i = 0; i < methods.length; i++) {
                    var description = methods[i].description,
                        id = methods[i].id,
                        tmplString,
                        renderedTmpl;

                    tmplString = mboSettingsObj.unity
                        ? '<div class="col-xs-1 MethodsEntryCheckBox"><input class="ep-js" type="checkbox" name="MyMethod" id="MyMethod${id}" value="${id}" data-js=".uiValidate({required:true, checkBoxOptions: {validateAsGroup:true, minimumNumberAsGroup:1}})" /></div><div class="col-xs-11 MethodsEntryDesc"><label class="ep-label" for="MyMethod${id}">${description}</label></div>'
                        : '<tr class="MethodsEntryCheckBox"><td><input class="ep-js AlignTop" type="checkbox" name="MyMethod" id="${id}" value="${id}" data-js=".uiValidate({required:true, checkBoxOptions: {validateAsGroup:true, minimumNumberAsGroup:1}})" /></td><td class="MethodsEntryDesc DisplayInlineBlock"><label class="ep-label" for="${id}">${description}</label></td></tr>';

                    renderedTmpl = $.tmpl(
                        tmplString,
                        {
                            id: id,
                            description: description
                        }
                    );

                    if ( methodsListElem ) {
                        methodsListElem.append(renderedTmpl);
                    }
                }

                if ( methodsListElem ) {
                    methodsListElem.find('input[name=MyMethod]').metaparse();
                    methodsListElem.closest('.DetailsRow').show();
                    if (methods.length === 0) {
                        methodsListElem.find('.NoMethodsFound').show();
                        methodsListElem.find('.SepaWarning').hide();
                    } else {
                        methodsListElem.find('.SepaWarning').show();
                    }
                }
            },

            /*
             * method to populate profiles select field with handed over profiles
             */
            addProfilesToSelect = function(profiles) {
                var option;

                // loop over profiles array and create for each profile an option
                for (var i = 0; i < profiles.length; i++) {
                    option = document.createElement("option");

                    option.text = profiles[i].id + " - " + profiles[i].name + " (" + profiles[i].website  +  ")";
                    option.value = profiles[i].id;

                    mboSettingsObj.profileSelect.add(option);
                }

                mboSettingsObj.profileSelect.disabled = false;
            },

            /*
             * method to prepare this module
             */
            setup = function() {
                // register events for profile select field
                mboSettingsObj.profileSelect && mboSettingsObj.profileSelect.addEventListener("change", function() {
                    onChangeProfileId(this);
                });

                // register events for test mode select field
                mboSettingsObj.testModeSelect && mboSettingsObj.testModeSelect.addEventListener("change", function() {
                    onChangeTestMode(this);
                });

                // previous profile button exists
                if (mboSettingsObj.previousMollieProfiles) {
                    // register events for previous profile button
                    mboSettingsObj.previousMollieProfiles.addEventListener("click", function() {
                        onClickGetProfileList('previous');
                    });
                    mboSettingsObj.$previousMollieProfiles = $(mboSettingsObj.previousMollieProfiles);
                }

                // previous profile button exists
                if (mboSettingsObj.previousMollieProfiles) {
                    // register events for next profile button
                    mboSettingsObj.nextMollieProfiles && mboSettingsObj.nextMollieProfiles.addEventListener("click", function() {
                        onClickGetProfileList('next');
                    });
                    mboSettingsObj.$nextMollieProfiles = $(mboSettingsObj.nextMollieProfiles);
                }
            };

    /*
     * @params
     *
     * - options - (object) setup options for the mbosettings module
     *             {
     *               profilesList: (json) JSON list of profiles
     *               profileSelect: (DOM node) select field to chose profile,
     *               testModeSelect: (DOM node) select field to chose working mode (test/live),
     *               previousMollieProfiles: (DOM node) button to traverse profiles,
     *               nextMollieProfiles: (DOM node) button to traverse profiles,
     *               objectId: (number) object id of Mollie,
     *               methodsList: (DOM node) the div into which payment method checkboxes are added
     *               errorMessages: {
     *                 unknownError: message of unknown error
     *                 connectionError: message of connection error
     *               }
     *             }
     */
    return function(options) {
        var previouslySelectedMethod = "-1",
            mboSettingsObjDefault = {
            //  profileSelect: undefined,
            //  testModeSelect: undefined,
            //  previousMollieProfiles: undefined,
            //  nextMollieProfiles: undefined,
            //  nextProfileId: undefined,
            //  previousProfileId: undefined,
            //  objectId: undefined,
            //  methodsList: undefined,
                onChangeProfileId: onChangeProfileId,
                onChangeTestMode: onChangeTestMode,
                onClickGetProfileList: onClickGetProfileList,
                previouslySelectedMethod: "-1",
                errorMessages: {
                    unknownError: "unknownError",
                    connectionError: "connectionError"
                },
                unity: ep.config.unity
            };

        // extend default settings object with configuration options
        $.extend(mboSettingsObj, mboSettingsObjDefault, options);

        // setup module
        setup();

        return mboSettingsObj;
    };
});